import click
import sys
import json
from time import time

if sys.platform == "linux":
    from .linux import (
        python,
        java,
        pypy,
        exe
    )
elif sys.platform == "win32":
    from .windows import (
        python,
        java,
        pypy,
        exe
    )
else:
    raise Exception


@click.command("competest",
               context_settings={
                   "help_option_names": ['-h', '--help']
               })
@click.argument("language", type=click.Choice(["python", "java", "pypy", "exe"]), required=True)
@click.argument("program_file", type=click.Path(exists=True), required=True)
@click.option("--test-cases", "-t", type=click.File("r"), help="File containing test cases.", required=True)
def competest(language, program_file, test_cases):
    """Run PROGRAM_FILE with test cases from the file specified in --test-cases (or -t) option and check them against the correct output specified in the same file.

       Supported Languages: java, python, pypy and exe(i.e. compiled executables)"""
    test_cases = json.loads(test_cases.read())
    total_cases = len(test_cases)
    failed_cases = 0
    languages = {
        "python": python,
        "java": java,
        "pypy": pypy,
        "exe": exe
    }
    for i, test_case in enumerate(test_cases, 1):
        input_data = "\n".join(test_case["input"]).encode()
        required_output = "\n".join(test_case["output"])
        start_time = time()
        process = languages[language](program_file, input_data)
        time_taken = time()-start_time
        actual_output = process.stdout.decode().strip().replace('\r', '')
        if process.returncode == 0 and required_output == actual_output:
            print(f"Test Case {i} Passed and took {time_taken:.3f} seconds  ✅")
        else:
            failed_cases += 1
            print(f"Test Case {i} Failed ❎")
            if process.returncode == 0:
                print(f"Required Output:\n{required_output}\n---------------")
                print(
                    f"Actual Output:\n{actual_output}\n---------------\n(took {time_taken:.3f} seconds)\n")
            else:
                print("---------------")
                print(f"Runtime Error: (took {time_taken:.3f} seconds)")
                print(process.stderr.decode().strip())
                print("---------------\n")

    if failed_cases == 0:
        print("All tests passed successfully. ✅")
    else:
        print(f"{failed_cases}/{total_cases} Test Cases Failed ❎")



import subprocess
import pathlib
from click import BadParameter


def python(program_file, input_data):
    return subprocess.run(["python3", program_file],
                          input=input_data,
                          capture_output=True)


def pypy(program_file, input_data):
    return subprocess.run(["pypy3", program_file],
                          input=input_data,
                          capture_output=True)


def java(program_file, input_data):
    program_file = pathlib.Path(program_file).resolve()
    if program_file.suffix != ".java":
        raise BadParameter(
            f"for java PROGRAM_FILE extention should be .java not {program_file.suffix=}")
    classname = program_file.stem
    program_path = program_file.parent
    subprocess.run(["javac", program_file])
    return subprocess.run(["java", "-classpath", program_path, classname],
                          input=input_data,
                          capture_output=True)


def exe(program_file, input_data):
    return subprocess.run([program_file],
                          input=input_data,
                          capture_output=True)



from setuptools import setup, find_packages

setup(
    name='competest',
    version='0.0.1',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Click',
    ],
    entry_points='''
        [console_scripts]
        competest=competest.competest:competest
    ''',
    author="Shailesh Aanand",
    author_email="anaandshailu@gmail.com",
    description="A python cli tool to test a program against test cases for some desired output."
)
